//
//  PhobosDataManager.m
//  GMCache
//
//  Created by Locus on 2020/1/21.
//

#import "PhobosDataManager.h"
#import "PhobosSendManager.h"
#import "PhobosConfig.h"
#import "PhobosUtil.h"
#import <MJExtension/MJExtension.h>
#import "NSManagedObjectModel+MagicalRecord.h"
#import "NSPersistentStoreCoordinator+MagicalRecord.h"

@implementation PhobosDataManager

static NSManagedObjectContext *Phobos_defaultContext;

+ (void)initialize {
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        NSBundle *bundle = [NSBundle bundleForClass:[self class]];
        NSURL *modelURL = [bundle URLForResource:@"GMCoreData" withExtension:@"momd"];//NewCodeDataModel.xcdatamodeld
        NSManagedObjectModel *model = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
        [NSManagedObjectModel MR_setDefaultManagedObjectModel:model];
        [MagicalRecord setupAutoMigratingCoreDataStack];
    });
}

+ (NSArray<SendDataEntity *> *)fetchDataEntitiesWithImmediately:(BOOL)immediately {
    NSPredicate *predicate;
    if (immediately) {
        predicate = [NSPredicate predicateWithFormat:@"status = %d or status = %d", PhobosDataSendStatusToBeSend, PhobosDataSendStatusError];
    } else {
        predicate = [NSPredicate predicateWithFormat:@"status = %d", PhobosDataSendStatusNormal];
    }
    return [self fetchDataEntitiesWithPredicate:predicate];
}

+ (NSArray<SendDataEntity *> *)fetchDataEntitiesWithPredicate:(NSPredicate *)searchFilter {
    return [SendDataEntity MR_findAllWithPredicate:searchFilter inContext:[NSManagedObjectContext MR_defaultContext]];
}

+ (NSUInteger)fetchCountOfEntitiesWithPredicate:(NSPredicate *)searchFilter {
    return [SendDataEntity MR_numberOfEntitiesWithPredicate:searchFilter inContext:[NSManagedObjectContext MR_defaultContext]];
}

+ (void)insertData:(NSDictionary *)data sendAPI:(NSString *)sendAPI immediately:(BOOL)immediately completion:(MRSaveCompletionHandler)completion {
    if (!sendAPI || [sendAPI isEqualToString:@""] || !data) {
        return;
    }
    SendDataEntity *entity = [SendDataEntity MR_createEntity];
    entity.data = [data mj_JSONData];
    entity.api = sendAPI;
    entity.status = immediately ? PhobosDataSendStatusToBeSend : PhobosDataSendStatusNormal;
    entity.id = [PhobosUtil currentMMTime];
    [[NSManagedObjectContext MR_defaultContext] MR_saveWithOptions:MRSaveSynchronously completion:completion];
}

+ (void)updateDataEntities:(NSArray<SendDataEntity *> *)entities sendStatus:(PhobosDataSendStatus)sendStatus completion:(MRSaveCompletionHandler)completion {
    if (entities.count > 0) {
        [entities enumerateObjectsUsingBlock:^(SendDataEntity *obj, NSUInteger idx, BOOL * _Nonnull stop) {
            obj.status = sendStatus;
        }];
        [[NSManagedObjectContext MR_defaultContext] MR_saveWithOptions:MRSaveSynchronously completion:completion];
    }
}

+ (void)deleteDataEntities:(NSArray<SendDataEntity *> *)entities {
    if (entities.count > 0) {
        [entities enumerateObjectsUsingBlock:^(SendDataEntity *obj, NSUInteger idx, BOOL * _Nonnull stop) {
            [obj MR_deleteEntity];
        }];
        [[NSManagedObjectContext MR_defaultContext] MR_saveOnlySelfAndWait];
    }
}

@end
